import { Pool } from "mariadb";
import { AbstractBatchEntityReaderStream, AbstractBatchEntityReaderStreamOptions } from "../../common/index";
import { BatchData, ReadCallback } from "batchjs";
/**
 * @interface
 * Options for the MariadbBatchEntityReader.
 * @extends AbstractBatchEntityReaderStreamOptions
 */
export interface MariadbBatchEntityReaderOptions extends AbstractBatchEntityReaderStreamOptions {
    pool: Pool;
    query: string;
}
/**
 * @class
 * Class that read data in batches of a specified size in Mariadb databases.
 * @extends AbstractBatchEntityReaderStream
 * @template T input chunk
 * @template E row entity
 */
export declare abstract class MariadbBatchEntityReader<T, E> extends AbstractBatchEntityReaderStream<T> {
    private readonly pool;
    private readonly query;
    private client;
    private fetchEntityStatement;
    private entitiesRead;
    /**
     * @constructor
     * @param {MariadbBatchEntityReaderOptions} options - The options for the MariadbBatchEntityReader.
     * @param [options.pool] {Pool} - The MariadbQL connection pool.
     * @param [options.query] {string} - SQL query to be executed (without LIMIT and OFFSET).
     */
    constructor(options: MariadbBatchEntityReaderOptions);
    /**
     * Fetches a batch of data from the database.
     *
     * @private
     * @param {number} size - The size of the batch to fetch.
     * @returns {Promise<BatchData<T>>} A promise that resolves with the batch of data.
     */
    protected fetch(size: number): Promise<BatchData<T>>;
    /**
     * Destroys the writer by finalizing the statement used to read entities and
     * closing the database connection. This method should be called when the
     * writer is no longer needed to free up resources.
     * @see AbstractBatchEntityReaderStream._destroy
     * @private
     * @param error {Error|null} - The error that caused the destruction.
     * @param callback {ReadCallback} - The callback function to be executed after destroying the reader.
     */
    _destroy(error: Error | null, callback: ReadCallback): void;
    /**
     * Connects to the database by creating a new database connection if none
     * already exists, or by reusing an existing connection.
     * @private
     * @returns {Promise<PoolConnection>} A promise that resolves with the database connection.
     */
    private connectDatabase;
    /**
     * Disconnects from the database by closing the active database connection
     * and setting the connection reference to null. This method should be called
     * when the reader is no longer needed to free up resources.
     * @returns {Promise<void>} A promise that resolves when the database connection
     * is successfully closed.
     */
    private disconnectDatabase;
    /**
     * Prepares a statement for fetching entities. If the statement has already been
     * prepared, it is reused.
     * @private
     * @param {Mariadb.Database} db - The database connection.
     * @returns {Promise<Mariadb.Statement>} The prepared statement.
     */
    private prepareStatement;
    /**
     * Finalizes the statement used to fetch entities. This method should be
     * called when the reader is no longer needed to free up resources.
     *
     * @private
     */
    private finalizeStatement;
    /**
     * Abstract method to convert a row to an entity. This method should be implemented
     * by subclasses to define the specific logic for reading a batch of data.
     *
     * @abstract
     * @protected
     * @param row {E} - The row to be converted to an entity.
     */
    protected abstract rowToEntity(row: E): T;
}
