"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MariadbBatchEntityReader = void 0;
const index_1 = require("../../common/index");
/**
 * @class
 * Class that read data in batches of a specified size in Mariadb databases.
 * @extends AbstractBatchEntityReaderStream
 * @template T input chunk
 * @template E row entity
 */
class MariadbBatchEntityReader extends index_1.AbstractBatchEntityReaderStream {
    /**
     * @constructor
     * @param {MariadbBatchEntityReaderOptions} options - The options for the MariadbBatchEntityReader.
     * @param [options.pool] {Pool} - The MariadbQL connection pool.
     * @param [options.query] {string} - SQL query to be executed (without LIMIT and OFFSET).
     */
    constructor(options) {
        super(options);
        this.client = null;
        this.fetchEntityStatement = null;
        this.entitiesRead = 0;
        this.pool = options.pool;
        this.query = options.query;
    }
    /**
     * Fetches a batch of data from the database.
     *
     * @private
     * @param {number} size - The size of the batch to fetch.
     * @returns {Promise<BatchData<T>>} A promise that resolves with the batch of data.
     */
    async fetch(size) {
        return this.connectDatabase()
            .then((client) => this.prepareStatement(client))
            .then((statement) => statement.execute([size, this.entitiesRead]))
            .then((results) => {
            this.entitiesRead += size;
            return results.map(this.rowToEntity);
        });
    }
    /**
     * Destroys the writer by finalizing the statement used to read entities and
     * closing the database connection. This method should be called when the
     * writer is no longer needed to free up resources.
     * @see AbstractBatchEntityReaderStream._destroy
     * @private
     * @param error {Error|null} - The error that caused the destruction.
     * @param callback {ReadCallback} - The callback function to be executed after destroying the reader.
     */
    _destroy(error, callback) {
        let destroyError = error;
        this.finalizeStatement()
            .then(() => this.disconnectDatabase())
            .catch((error) => { destroyError = error; })
            .finally(() => super._destroy(destroyError, callback));
    }
    /**
     * Connects to the database by creating a new database connection if none
     * already exists, or by reusing an existing connection.
     * @private
     * @returns {Promise<PoolConnection>} A promise that resolves with the database connection.
     */
    async connectDatabase() {
        if (!this.client) {
            this.client = await this.pool.getConnection();
        }
        return this.client;
    }
    /**
     * Disconnects from the database by closing the active database connection
     * and setting the connection reference to null. This method should be called
     * when the reader is no longer needed to free up resources.
     * @returns {Promise<void>} A promise that resolves when the database connection
     * is successfully closed.
     */
    async disconnectDatabase() {
        if (this.client) {
            await this.client.release();
            this.client = null;
        }
    }
    /**
     * Prepares a statement for fetching entities. If the statement has already been
     * prepared, it is reused.
     * @private
     * @param {Mariadb.Database} db - The database connection.
     * @returns {Promise<Mariadb.Statement>} The prepared statement.
     */
    async prepareStatement(client) {
        if (!this.fetchEntityStatement) {
            this.fetchEntityStatement = await client.prepare(`${this.query} LIMIT ? OFFSET ?`);
        }
        return this.fetchEntityStatement;
    }
    /**
     * Finalizes the statement used to fetch entities. This method should be
     * called when the reader is no longer needed to free up resources.
     *
     * @private
     */
    async finalizeStatement() {
        if (this.fetchEntityStatement) {
            this.fetchEntityStatement.close();
            this.fetchEntityStatement = null;
        }
    }
}
exports.MariadbBatchEntityReader = MariadbBatchEntityReader;
