"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MysqlBatchEntityReader = void 0;
const index_1 = require("../../common/index");
/**
 * @class
 * Class that reads data in batches of a specified size from a MySQL database.
 * @extends AbstractBatchEntityReaderStream
 * @template T input chunk
 * @template E row entity
 */
class MysqlBatchEntityReader extends index_1.AbstractBatchEntityReaderStream {
    /**
     * @constructor
     * @param {MysqlBatchEntityReaderOptions} options - The options for the MysqlBatchEntityReader.
     * @param [options.pool] {Pool} - The MySQL connection pool.
     * @param [options.query] {string} - SQL query to be executed (without LIMIT and OFFSET).
     */
    constructor(options) {
        super(options);
        this.client = null;
        this.entitiesRead = 0;
        this.pool = options.pool;
        this.query = options.query;
    }
    /**
     * Fetches a batch of data from the database.
     *
     * @private
     * @param {number} size - The size of the batch to fetch.
     * @returns {Promise<BatchData<T>>} A promise that resolves with the batch of data.
     */
    async fetch(size) {
        return this.connectDatabase()
            .then((client) => client.query(`${this.query} LIMIT ${size} OFFSET ${this.entitiesRead}`))
            .then(([results]) => {
            this.entitiesRead += size;
            return results.map(this.rowToEntity);
        });
    }
    /**
     * Destroys the writer by closing the database connection. This method should be called when the
     * writer is no longer needed to free up resources.
     * @see AbstractBatchEntityReaderStream._destroy
     * @private
     * @param error {Error|null} - The error that caused the destruction.
     * @param callback {ReadCallback} - The callback function to be executed after destroying the reader.
     */
    _destroy(error, callback) {
        let destroyError = error;
        this.disconnectDatabase()
            .catch((err) => (destroyError = err))
            .finally(() => super._destroy(destroyError, callback));
    }
    /**
     * Connects to the database by creating a new database connection if none
     * already exists, or by reusing an existing connection.
     * @private
     * @returns {Promise<PoolConnection>} A promise that resolves with the database connection.
     */
    async connectDatabase() {
        if (!this.client) {
            this.client = await this.pool.getConnection();
        }
        return this.client;
    }
    /**
     * Disconnects from the database by releasing the active database connection
     * and setting the client reference to null. This method should be called
     * when the reader is no longer needed to free up resources.
     * @returns {Promise<void>} A promise that resolves when the database connection
     * is successfully released.
     */
    disconnectDatabase() {
        if (this.client) {
            this.client.release();
            this.client = null;
        }
        return Promise.resolve();
    }
}
exports.MysqlBatchEntityReader = MysqlBatchEntityReader;
