"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MysqlBatchEntityWriter = void 0;
const common_1 = require("../../common");
/**
 * @class
 * Class that writes data in batches of a specified size into a MySQL database.
 * @extends AbstractBatchEntityWriterStream
 * @template T input chunk
 * @template E row entity
 */
class MysqlBatchEntityWriter extends common_1.AbstractBatchEntityWriterStream {
    /**
     * @constructor
     * @param {MysqlBatchEntityWriterOptions} options - The options for the MysqlBatchEntityWriter.
     * @param [options.pool] {Pool} - The MySQL connection pool.
     * @param [options.prepareStatement] {String} - Insert SQL prepared statement to be executed.
     */
    constructor(options) {
        super(options);
        this.pool = options.pool;
        this.prepareStatement = options.prepareStatement;
    }
    /**
     * Writes a batch of data to the storage.
     *
     * @protected
     * @param {BatchData<T>} chunk - The batch of data to write to the storage.
     * @returns {Promise<void>} A promise that resolves when the batch is successfully written.
     *                          The promise should be rejected if there is an error during writing.
     */
    async batchWrite(chunk) {
        const client = await this.pool.getConnection();
        try {
            await client.beginTransaction();
            await Promise.all(chunk.map((entity) => client.query(this.prepareStatement, this.entityToRow(entity))));
            await client.commit();
        }
        catch (error) {
            await client.rollback();
            throw error;
        }
        finally {
            client.release();
        }
    }
}
exports.MysqlBatchEntityWriter = MysqlBatchEntityWriter;
