"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqliteBatchEntityReader = void 0;
const index_1 = require("../../common/index");
/**
 * @class
 * Class that read data in batches of a specified size in SQLite databases.
 * @extends AbstractBatchEntityReaderStream
 * @template T
 */
class SqliteBatchEntityReader extends index_1.AbstractBatchEntityReaderStream {
    /**
     * @constructor
     * @param {SqliteBatchEntityReaderOptions} options - The options for the SqliteBatchEntityReader.
     * @param [options.dbConnectionFactory] {Function} - Function that creates a database connection.
     */
    constructor(options) {
        super(options);
        this.dbConnection = null;
        this.fetchEntityStatement = null;
        this.entitiesRead = 0;
        this.dbConnectionFactory = options.dbConnectionFactory;
        this.query = options.query;
    }
    /**
     * Fetches a batch of data from the database.
     *
     * @private
     * @param {number} size - The size of the batch to fetch.
     * @returns {Promise<BatchData<T>>} A promise that resolves with the batch of data.
     */
    async fetch(size) {
        return this.connectDatabase()
            .then((db) => this.prepareStatement(db))
            .then((statement) => statement.all({ "@limit": size, "@offset": this.entitiesRead }))
            .then((results) => {
            this.entitiesRead += size;
            return results.map(this.rowToEntity);
            ;
        });
    }
    /**
     * Destroys the writer by finalizing the statement used to read entities and
     * closing the database connection. This method should be called when the
     * writer is no longer needed to free up resources.
     * @see AbstractBatchEntityReaderStream._destroy
     * @private
     * @param error {Error|null} - The error that caused the destruction.
     * @param callback {ReadCallback} - The callback function to be executed after destroying the reader.
     */
    _destroy(error, callback) {
        let destroyError = error;
        this.finalizeStatement()
            .then(() => this.disconnectDatabase())
            .catch((error) => { destroyError = error; })
            .finally(() => super._destroy(destroyError, callback));
    }
    /**
     * Connects to the database by creating a new database connection if none
     * already exists, or by reusing an existing connection.
     * @private
     * @returns {Promise<sqlite.Database>} A promise that resolves with the database connection.
     */
    async connectDatabase() {
        if (!this.dbConnection) {
            this.dbConnection = await this.dbConnectionFactory();
        }
        return this.dbConnection;
    }
    /**
     * Disconnects from the database by closing the active database connection
     * and setting the connection reference to null. This method should be called
     * when the reader is no longer needed to free up resources.
     * @returns {Promise<void>} A promise that resolves when the database connection
     * is successfully closed.
     */
    async disconnectDatabase() {
        if (this.dbConnection) {
            await this.dbConnection.close();
            this.dbConnection = null;
        }
    }
    /**
     * Prepares a statement for fetching entities. If the statement has already been
     * prepared, it is reused.
     * @private
     * @param {sqlite.Database} db - The database connection.
     * @returns {Promise<sqlite.Statement>} The prepared statement.
     */
    async prepareStatement(db) {
        if (!this.fetchEntityStatement) {
            this.fetchEntityStatement = await db.prepare(`${this.query} LIMIT @limit OFFSET @offset`);
        }
        return this.fetchEntityStatement;
    }
    /**
     * Finalizes the statement used to fetch entities. This method should be
     * called when the reader is no longer needed to free up resources.
     *
     * @private
     */
    async finalizeStatement() {
        if (this.fetchEntityStatement) {
            await this.fetchEntityStatement.finalize();
            this.fetchEntityStatement = null;
        }
    }
}
exports.SqliteBatchEntityReader = SqliteBatchEntityReader;
