"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqliteBatchEntityWriter = void 0;
const index_1 = require("../../common/index");
/**
 * @class
 * Class that write data in batches of a specified size in SQLite databases.
 * @extends AbstractBatchEntityWriterStream
 * @template T
 */
class SqliteBatchEntityWriter extends index_1.AbstractBatchEntityWriterStream {
    /**
     * @constructor
     * @param {SqliteBatchEntityWriterOptions} options - The options for the SqliteBatchEntityWriter.
     * @param [options.dbConnectionFactory] {Function} - Function that creates a database connection.
     */
    constructor(options) {
        super(options);
        this.saveEntityStatement = null;
        this.dbConnectionFactory = options.dbConnectionFactory;
        this.prepareStatementString = options.prepareStatement;
    }
    /**
     * Writes a batch of data to the storage.
     *
     * @protected
     * @param {BatchData<T>} chunk - The batch of data to write to the storage.
     * @returns {Promise<void>} A promise that resolves when the batch is successfully written.
     *                           The promise should be rejected if there is an error during writing.
     */
    async batchWrite(chunk) {
        const db = await this.dbConnectionFactory();
        return db.exec("BEGIN TRANSACTION")
            .then(() => this.executeBatch(db, chunk))
            .then(() => this.commitTransaction(db))
            .catch((error) => {
            this.rollbackTransaction(db);
            return Promise.reject(error);
        }).finally(async () => {
            await this.finalizeStatement();
            await db.close();
        });
    }
    /**
     * Executes a batch of data in the database, commits the transaction if all
     * promises are resolved, or rolls back the transaction if any of the promises
     * are rejected.
     *
     * @private
     * @param {sqlite3.Database} db - The database connection.
     * @param {BatchData<T>} chunk - The batch of data to write to the storage.
     * @returns {Promise<void[]>} A promise that resolves when all promises are resolved.
     */
    async executeBatch(db, chunk) {
        const stmt = await this.prepareStatement(db);
        const promises = chunk.map((entity) => this.saveEntity(entity, stmt));
        return Promise.all(promises);
    }
    /**
     * Commits the transaction if no errors occurred during the batch execution.
     * If an error occurred, the transaction is rolled back and the error is propagated.
     *
     * @private
     * @param {sqlite.Database} db - The database connection.
     * @returns {Promise<void>} A promise that resolves when the transaction is committed.
     */
    async commitTransaction(db) {
        return db.exec("COMMIT");
    }
    /**
     * Rolls back the transaction if an error occurred during the batch execution.
     * The error is propagated to the caller.
     *
     * @private
     * @param {sqlite.Database} db - The database connection.
     * @returns {Promise<void>} A promise that resolves when the transaction is rolled back.
     */
    async rollbackTransaction(db) {
        return db.exec("ROLLBACK");
    }
    /**
     * Prepares a statement for saving entities. If the statement has already been
     * prepared, it is reused.
     * @private
     * @param {sqlite.Database} db - The database connection.
     * @returns {Promise<sqlite.Statement>} The prepared statement.
     */
    async prepareStatement(db) {
        if (!this.saveEntityStatement) {
            this.saveEntityStatement = await db.prepare(this.prepareStatementString);
        }
        return this.saveEntityStatement;
    }
    /**
     * Finalizes the statement used to save entities. This method should be
     * called when the writer is no longer needed to free up resources.
     *
     * @private
     */
    async finalizeStatement() {
        if (this.saveEntityStatement) {
            await this.saveEntityStatement.finalize();
            this.saveEntityStatement = null;
        }
    }
    /**
     * Finalizes the writer by calling the _final method of the superclass and
     * finalizing the statement used to save entities. This method should be
     * called when the writer is no longer needed to free up resources.
     * @see AbstractBatchEntityWriterStream._final
     * @private
     * @param callback {WriteCallback} - The callback function to be executed after finalizing the writer.
     * @returns {Promise<void>}
     */
    async _final(callback) {
        try {
            await super._final(callback);
        }
        finally {
            await this.finalizeStatement();
        }
    }
}
exports.SqliteBatchEntityWriter = SqliteBatchEntityWriter;
