import { AbstractBatchEntityReaderStream } from "../../common/index";
/**
 * @class
 * Class that reads data in batches of a specified size using PostgreSQL cursors.
 * @extends AbstractBatchEntityReaderStream
 * @template T
 */
export class PostgresBatchEntityReader extends AbstractBatchEntityReaderStream {
    /**
     * @constructor
     * @param {PostgresBatchEntityReaderOptions} options - The options for the PostgresBatchEntityReader.
     * @param [options.pool] {Pool} - The PostgreSQL connection pool.
     * @param [options.query] {string} - SQL query to be executed (without LIMIT and OFFSET).
     */
    constructor(options) {
        super(options);
        this.client = null;
        this.cursorName = null;
        this.cursorOpened = false;
        this.pool = options.pool;
        this.query = options.query;
    }
    /**
     * Fetches a batch of data using a PostgreSQL cursor.
     *
     * @protected
     * @param {number} size - The size of the batch to fetch.
     * @returns {Promise<BatchData<T>>} A promise that resolves with the batch of data.
     */
    fetch(size) {
        return this.initializeCursor()
            .then(({ cursorName, client }) => client.query({
            text: `FETCH ${size} FROM ${cursorName};`,
        }))
            .then((result) => result.rows.map(this.rowToEntity));
    }
    /**
     * Initializes the cursor for the query if not already initialized.
     *
     * @private
     * @returns {Promise<{cursorName:string,client:PoolClient}>} A promise that resolves when the cursor is initialized.
     */
    async initializeCursor() {
        if (!this.cursorOpened) {
            this.cursorName = `cursor_${Date.now()}`;
            return this.pool.connect()
                .then((client) => {
                this.client = client;
                return this.client.query("BEGIN;");
            })
                .then(() => this.client.query(`DECLARE ${this.cursorName} CURSOR FOR ${this.query};`))
                .then(() => {
                this.cursorOpened = true;
            })
                .then(() => ({ cursorName: this.cursorName, client: this.client }));
        }
        return Promise.resolve({ cursorName: this.cursorName, client: this.client });
    }
    /**
     * Closes the cursor and releases the client connection.
     *
     * @private
     * @returns {Promise<void>} A promise that resolves when the cursor is closed.
     */
    async closeCursor() {
        if (this.cursorOpened && this.cursorName && this.client) {
            try {
                await this.client.query(`CLOSE ${this.cursorName};`);
                await this.client.query("COMMIT;");
            }
            finally {
                this.client.release();
                this.client = null;
                this.cursorName = null;
                this.cursorOpened = false;
            }
        }
    }
    /**
     * Destroys the reader by closing the cursor and releasing resources.
     *
     * @param {Error|null} error - The error that caused the destruction.
     * @param {Function} callback - The callback function to be executed after destruction.
     */
    _destroy(error, callback) {
        let destroyError = error;
        this.closeCursor()
            .catch((error) => { destroyError = error; })
            .finally(() => super._destroy(destroyError, callback));
    }
}
